@@ -1,4 +1,13 @@
 # Testing TSAdjoint and matrix-free Jacobian
+# Basic usage:
+#     python vanderpol.py
+# Test implicit methods using implicit form:
+#     python -implicitform
+# Test explicit methods:
+#     python -implicitform 0
+# Test IMEX methods:
+#     python -imexform
+# Matrix-free implementations can be enabled with an additional option -mf
 
 import sys, petsc4py
 petsc4py.init(sys.argv)
@@ -8,14 +17,19 @@ from petsc4py import PETSc
 class VDP(object):
     n = 2
     comm = PETSc.COMM_SELF
-    def __init__(self, mu_=1.0e3,mf_=False):
+    def __init__(self, mu_=1.0e3, mf_=False, imex_=False):
         self.mu_ = mu_
         self.mf_ = mf_
+        self.imex_ = imex_
         if self.mf_:
-            self.J_ = PETSc.Mat().createDense([self.n,self.n], comm=self.comm)
-            self.J_.setUp()
-            self.Jp_ = PETSc.Mat().createDense([self.n,1], comm=self.comm)
-            self.Jp_.setUp()
+            self.Jim_ = PETSc.Mat().createDense([self.n,self.n], comm=self.comm)
+            self.Jim_.setUp()
+            self.JimP_ = PETSc.Mat().createDense([self.n,1], comm=self.comm)
+            self.JimP_.setUp()
+            self.Jex_ = PETSc.Mat().createDense([self.n,self.n], comm=self.comm)
+            self.Jex_.setUp()
+            self.JexP_ = PETSc.Mat().createDense([self.n,1], comm=self.comm)
+            self.JexP_.setUp()
     def initialCondition(self, u):
         mu = self.mu_
         u[0] = 2.0
@@ -24,18 +38,25 @@ class VDP(object):
     def evalFunction(self, ts, t, u, f):
         mu = self.mu_
         f[0] = u[1]
-        f[1] = mu*((1.-u[0]*u[0])*u[1]-u[0])
+        if self.imex_:
+            f[1] = 0.0
+        else:
+            f[1] = mu*((1.-u[0]*u[0])*u[1]-u[0])
         f.assemble()
     def evalJacobian(self, ts, t, u, A, B):
         if not self.mf_:
             J = A
         else :
-            J = self.J_
+            J = self.Jex_
         mu = self.mu_
         J[0,0] = 0
-        J[1,0] = -mu*(2.0*u[1]*u[0]+1.)
         J[0,1] = 1.0
-        J[1,1] = mu*(1.0-u[0]*u[0])
+        if self.imex_:
+            J[1,0] = 0
+            J[1,1] = 0
+        else:
+            J[1,0] = -mu*(2.0*u[1]*u[0]+1.)
+            J[1,1] = mu*(1.0-u[0]*u[0])
         J.assemble()
         if A != B: B.assemble()
         return True # same nonzero pattern
@@ -43,25 +64,33 @@ class VDP(object):
         if not self.mf_:
             Jp = C
         else:
-            Jp = self.Jp_
-        Jp[0,0] = 0
-        Jp[1,0] = (1.-u[0]*u[0])*u[1]-u[0]
-        Jp.assemble()
+            Jp = self.JexP_
+        if not self.imex_:
+            Jp[0,0] = 0
+            Jp[1,0] = (1.-u[0]*u[0])*u[1]-u[0]
+            Jp.assemble()
         return True
     def evalIFunction(self, ts, t, u, udot, f):
         mu = self.mu_
-        f[0] = udot[0]-u[1]
+        if self.imex_:
+            f[0] = udot[0]
+        else:
+            f[0] = udot[0]-u[1]
         f[1] = udot[1]-mu*((1.-u[0]*u[0])*u[1]-u[0])
         f.assemble()
     def evalIJacobian(self, ts, t, u, udot, shift, A, B):
         if not self.mf_:
             J = A
         else :
-            J = self.J_
+            J = self.Jim_
         mu = self.mu_
-        J[0,0] = shift
+        if self.imex_:
+            J[0,0] = shift
+            J[0,1] = 0.0
+        else:
+            J[0,0] = shift
+            J[0,1] = -1.0
         J[1,0] = mu*(2.0*u[1]*u[0]+1.)
-        J[0,1] = -1.0
         J[1,1] = shift-mu*(1.0-u[0]*u[0])
         J.assemble()
         if A != B: B.assemble()
@@ -70,7 +99,7 @@ class VDP(object):
         if not self.mf_:
             Jp = C
         else:
-            Jp = self.Jp_
+            Jp = self.JimP_
         Jp[0,0] = 0
         Jp[1,0] = u[0]-(1.-u[0]*u[0])*u[1]
         Jp.assemble()
@@ -81,46 +110,84 @@ class JacShell:
         self.ode_ = ode
     def mult(self, A, x, y):
         "y <- A * x"
-        self.ode_.J_.mult(x,y)
+        self.ode_.Jex_.mult(x,y)
     def multTranspose(self, A, x, y):
         "y <- A' * x"
-        self.ode_.J_.multTranspose(x, y)
+        self.ode_.Jex_.multTranspose(x, y)
 
 class JacPShell:
     def __init__(self, ode):
         self.ode_ = ode
     def multTranspose(self, A, x, y):
         "y <- A' * x"
-        self.ode_.Jp_.multTranspose(x, y)
+        self.ode_.JexP_.multTranspose(x, y)
+
+class IJacShell:
+    def __init__(self, ode):
+        self.ode_ = ode
+    def mult(self, A, x, y):
+        "y <- A * x"
+        self.ode_.Jim_.mult(x,y)
+    def multTranspose(self, A, x, y):
+        "y <- A' * x"
+        self.ode_.Jim_.multTranspose(x, y)
+
+class IJacPShell:
+    def __init__(self, ode):
+        self.ode_ = ode
+    def multTranspose(self, A, x, y):
+        "y <- A' * x"
+        self.ode_.JimP_.multTranspose(x, y)
+
 OptDB = PETSc.Options()
 
 mu_ = OptDB.getScalar('mu', 1.0e3)
 mf_ = OptDB.getBool('mf', False)
 
 implicitform_ = OptDB.getBool('implicitform', False)
+imexform_ = OptDB.getBool('imexform', False)
 
-ode = VDP(mu_,mf_)
+ode = VDP(mu_,mf_,imexform_)
 
 if not mf_:
-    J = PETSc.Mat().createDense([ode.n,ode.n], comm=ode.comm)
-    J.setUp()
-    Jp = PETSc.Mat().createDense([ode.n,1], comm=ode.comm)
-    Jp.setUp()
+    Jim = PETSc.Mat().createDense([ode.n,ode.n], comm=ode.comm)
+    Jim.setUp()
+    JimP = PETSc.Mat().createDense([ode.n,1], comm=ode.comm)
+    JimP.setUp()
+    Jex = PETSc.Mat().createDense([ode.n,ode.n], comm=ode.comm)
+    Jex.setUp()
+    JexP = PETSc.Mat().createDense([ode.n,1], comm=ode.comm)
+    JexP.setUp()
 else:
-    J = PETSc.Mat().create()
-    J.setSizes([ode.n,ode.n])
-    J.setType('python')
+    Jim = PETSc.Mat().create()
+    Jim.setSizes([ode.n,ode.n])
+    Jim.setType('python')
+    shell = IJacShell(ode)
+    Jim.setPythonContext(shell)
+    Jim.setUp()
+    Jim.assemble()
+    JimP = PETSc.Mat().create()
+    JimP.setSizes([ode.n,1])
+    JimP.setType('python')
+    shell = IJacPShell(ode)
+    JimP.setPythonContext(shell)
+    JimP.setUp()
+    JimP.assemble()
+    Jex = PETSc.Mat().create()
+    Jex.setSizes([ode.n,ode.n])
+    Jex.setType('python')
     shell = JacShell(ode)
-    J.setPythonContext(shell)
-    J.setUp()
-    J.assemble()
-    Jp = PETSc.Mat().create()
-    Jp.setSizes([ode.n,1])
-    Jp.setType('python')
+    Jex.setPythonContext(shell)
+    Jex.setUp()
+    Jex.assemble()
+    JexP = PETSc.Mat().create()
+    JexP.setSizes([ode.n,1])
+    JexP.setType('python')
     shell = JacPShell(ode)
-    Jp.setPythonContext(shell)
-    Jp.setUp()
-    Jp.assemble()
+    JexP.setPythonContext(shell)
+    JexP.setUp()
+    JexP.zeroEntries()
+    JexP.assemble()
 
 u = PETSc.Vec().createSeq(ode.n, comm=ode.comm)
 f = u.duplicate()
@@ -134,16 +201,25 @@ adj_p.append(PETSc.Vec().createSeq(1, comm=ode.comm))
 ts = PETSc.TS().create(comm=ode.comm)
 ts.setProblemType(ts.ProblemType.NONLINEAR)
 
-if implicitform_:
-    ts.setType(ts.Type.CN)
+if imexform_:
+    ts.setType(ts.Type.ARKIMEX)
     ts.setIFunction(ode.evalIFunction, f)
-    ts.setIJacobian(ode.evalIJacobian, J)
-    ts.setIJacobianP(ode.evalIJacobianP, Jp)
-else:
-    ts.setType(ts.Type.RK)
+    ts.setIJacobian(ode.evalIJacobian, Jim)
+    ts.setIJacobianP(ode.evalIJacobianP, JimP)
     ts.setRHSFunction(ode.evalFunction, f)
-    ts.setRHSJacobian(ode.evalJacobian, J)
-    ts.setRHSJacobianP(ode.evalJacobianP, Jp)
+    ts.setRHSJacobian(ode.evalJacobian, Jex)
+    ts.setRHSJacobianP(ode.evalJacobianP, JexP)
+else:
+    if implicitform_:
+        ts.setType(ts.Type.CN)
+        ts.setIFunction(ode.evalIFunction, f)
+        ts.setIJacobian(ode.evalIJacobian, Jim)
+        ts.setIJacobianP(ode.evalIJacobianP, JimP)
+    else:
+        ts.setType(ts.Type.RK)
+        ts.setRHSFunction(ode.evalFunction, f)
+        ts.setRHSJacobian(ode.evalJacobian, Jex)
+        ts.setRHSJacobianP(ode.evalJacobianP, JexP)
 
 ts.setSaveTrajectory()
 ts.setTime(0.0)
@@ -182,4 +258,4 @@ def compute_derp(du,dp):
 compute_derp(adj_u[0],adj_p[0])
 compute_derp(adj_u[1],adj_p[1])
 
-del ode, J, Jp, u, f, ts, adj_u, adj_p
+del ode, Jim, JimP, Jex, JexP, u, f, ts, adj_u, adj_p
