@@ -78,7 +78,7 @@ PetscErrorCode FormInitGuess(DM da, Vec X, Params *p)
   */
   PetscCall(DMDAVecRestoreArray(da,X,&x));
 
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 PetscErrorCode FormFunction(DM da, Vec X, Vec F, Params *p)
@@ -159,7 +159,7 @@ PetscErrorCode FormFunction(DM da, Vec X, Vec F, Params *p)
   PetscCall(DMRestoreLocalVector(da,&localX));
   PetscCall(PetscLogFlops(11.0*ym*xm));
 
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
 
 PetscErrorCode FormJacobian(DM da, Vec X, Mat J, Params *p)
@@ -258,5 +258,5 @@ PetscErrorCode FormJacobian(DM da, Vec X, Mat J, Params *p)
   */
   PetscCall(MatSetOption(J,MAT_NEW_NONZERO_LOCATION_ERR,PETSC_TRUE));
 
-  PetscFunctionReturn(0);
+  PetscFunctionReturn(PETSC_SUCCESS);
 }
