@@ -232,7 +232,19 @@ class TestPlex_1D(BaseTestPlex, unittest.TestCase):
     pass
 
 class TestPlex_2D(BaseTestPlex_2D, unittest.TestCase):
-    pass
+
+    def testTransform(self):
+            plex = self.plex
+            cstart, cend = plex.getHeightStratum(0)
+            tr = PETSc.DMPlexTransform().create(comm=PETSc.COMM_WORLD)
+            tr.setType(PETSc.DMPlexTransformType.REFINEALFELD)
+            tr.setDM(plex)
+            tr.setUp()
+            newplex = tr.apply(plex)
+            tr.destroy()
+            newcstart, newcend = newplex.getHeightStratum(0)
+            newplex.destroy()
+            self.assertTrue((newcend-newcstart) == 3*(cend-cstart)) 
 
 class TestPlex_3D(BaseTestPlex_3D, unittest.TestCase):
     pass
