@@ -1,7 +1,6 @@
 import unittest
 from petsc4py import PETSc
 from sys import getrefcount
-import gc
 
 # --------------------------------------------------------------------
 
@@ -31,9 +30,11 @@ class MyODE:
         if J != P: J.assemble()
         return False # same_nz
 
-class MyTS:
+class MyTS(object):
+
     def __init__(self):
         self.log = {}
+
     def _log(self, method, *args):
         self.log.setdefault(method, 0)
         self.log[method] += 1
@@ -83,9 +84,15 @@ class TestTSPython(unittest.TestCase):
         self.assertTrue('destroy' not in ctx.log)
         self.ts.destroy() # XXX
         self.ts = None
+        PETSc.garbage_cleanup()
         self.assertEqual(ctx.log['destroy'], 1)
         self.assertEqual(getrefcount(ctx),   2)
 
+    def testGetType(self):
+        ctx = self.ts.getPythonContext()
+        pytype = "{0}.{1}".format(ctx.__module__, type(ctx).__name__)
+        self.assertTrue(self.ts.getPythonType() == pytype)
+
     def testSolve(self):
         ts = self.ts
         ts.setProblemType(ts.ProblemType.NONLINEAR)
@@ -100,14 +107,13 @@ class TestTSPython(unittest.TestCase):
         ts.setIFunction(ode.function, f)
         ts.setIJacobian(ode.jacobian, J, J)
         ts.snes.ksp.pc.setType('none')
-        
+
         T0, dT, nT = 0.0, 0.1, 10
         T = T0 + nT*dT
         ts.setTime(T0)
         ts.setTimeStep(dT)
         ts.setMaxTime(T)
         ts.setMaxSteps(nT)
-        ts.setDuration(T, nT)
         ts.setFromOptions()
         u[0], u[1], u[2] = 1, 2, 3
         ts.solve(u)
@@ -115,7 +121,7 @@ class TestTSPython(unittest.TestCase):
 
         self.assertTrue(ode.function_calls > 0)
         self.assertTrue(ode.jacobian_calls > 0)
-        
+
         ctx = self.ts.getPythonContext()
         ncalls = self.nsolve * ts.step_number
         self.assertTrue(ctx.log['solveStep'] == ncalls)
@@ -146,7 +152,8 @@ class TestTSPython(unittest.TestCase):
         T = T0 + nT*dT
         ts.setTime(T0)
         ts.setTimeStep(dT)
-        ts.setDuration(T, nT)
+        ts.setMaxTime(T)
+        ts.setMaxSteps(nT)
         ts.setFromOptions()
         u[:] = 1, 2, 3, 4, 5
 
@@ -158,13 +165,22 @@ class TestTSPython(unittest.TestCase):
 
     def testResetAndSolve(self):
         self.ts.reset()
+        self.ts.setStepNumber(0)
         self.testSolve()
         self.ts.reset()
+        self.ts.setStepNumber(0)
         self.testFDColor()
         self.ts.reset()
+        self.ts.setStepNumber(0)
         self.testSolve()
         self.ts.reset()
 
+    def testSetAdaptLimits(self):
+        self.ts.setStepLimits(1.0, 2.0)
+        hmin, hmax = self.ts.getStepLimits()
+        self.assertEqual(1.0, hmin)
+        self.assertEqual(2.0, hmax)
+
 # --------------------------------------------------------------------
 
 if __name__ == '__main__':
